@@ -13,13 +13,15 @@ from typing import Union
 
 import six
 
+from ddtrace.internal.utils.deprecations import DDTraceDeprecationWarning
+
 from . import config
+from .constants import ANALYTICS_SAMPLE_RATE_KEY
 from .constants import ERROR_MSG
 from .constants import ERROR_STACK
 from .constants import ERROR_TYPE
 from .constants import MANUAL_DROP_KEY
 from .constants import MANUAL_KEEP_KEY
-from .constants import NUMERIC_TAGS
 from .constants import SERVICE_KEY
 from .constants import SERVICE_VERSION_KEY
 from .constants import SPAN_MEASURED_KEY
@@ -40,13 +42,15 @@ from .internal.compat import numeric_types
 from .internal.compat import stringify
 from .internal.compat import time_ns
 from .internal.logger import get_logger
-from .internal.utils.deprecation import deprecated
+from .vendor.debtcollector import deprecate
+from .vendor.debtcollector.removals import remove
 from .vendor.debtcollector.removals import removed_property
 
 
 if TYPE_CHECKING:
     from .tracer import Tracer
 
+_NUMERIC_TAGS = (ANALYTICS_SAMPLE_RATE_KEY,)
 
 _TagNameType = Union[Text, bytes]
 _MetaDictType = Dict[_TagNameType, Text]
@@ -67,11 +71,11 @@ class Span(object):
         "parent_id",
         "_meta",
         "error",
-        "metrics",
+        "_metrics",
         "_span_type",
         "start_ns",
         "duration_ns",
-        "tracer",
+        "_tracer",
         # Sampler attributes
         "sampled",
         # Internal attributes
@@ -105,8 +109,7 @@ class Span(object):
         that it was created in. Using a ``Span`` from within a child process
         could result in a deadlock or unexpected behavior.
 
-        :param ddtrace.Tracer tracer: the tracer that will submit this span when
-            finished.
+        :param ddtrace.Tracer tracer: deprecated. Pass ``None`` instead.
         :param str name: the name of the traced operation.
 
         :param str service: the service name
@@ -139,7 +142,7 @@ class Span(object):
         # tags / metadata
         self._meta = {}  # type: _MetaDictType
         self.error = 0
-        self.metrics = {}  # type: _MetricDictType
+        self._metrics = {}  # type: _MetricDictType
 
         # timing
         self.start_ns = time_ns() if start is None else int(start * 1e9)  # type: int
@@ -149,7 +152,15 @@ class Span(object):
         self.trace_id = trace_id or _rand.rand64bits()  # type: int
         self.span_id = span_id or _rand.rand64bits()  # type: int
         self.parent_id = parent_id  # type: Optional[int]
-        self.tracer = tracer  # type: Optional[Tracer]
+        self._tracer = None  # type: Optional[Tracer]
+        if tracer is not None:
+            deprecate(
+                "ddtrace.Span.tracer is deprecated",
+                message="Use Span(tracer=None, name, ...) instead.",
+                category=DDTraceDeprecationWarning,
+                removal_version="1.0.0",
+            )
+            self._tracer = tracer
         self._on_finish_callbacks = [] if on_finish is None else on_finish
 
         # sampling
@@ -167,6 +178,16 @@ class Span(object):
         else:
             self._ignored_exceptions.append(exc)
 
+    @removed_property(category=DDTraceDeprecationWarning, removal_version="1.0.0")
+    def tracer(self):
+        # type: () -> Optional[Tracer]
+        return self._tracer
+
+    @tracer.setter  # type: ignore
+    def tracer(self, t):
+        # type: (Optional[Tracer]) -> None
+        self._tracer = t
+
     @property
     def start(self):
         # type: () -> float
@@ -289,7 +310,7 @@ class Span(object):
             return
 
         # Key should explicitly be converted to a float if needed
-        elif key in NUMERIC_TAGS:
+        elif key in _NUMERIC_TAGS:
             if value is None:
                 log.debug("ignoring not number metric %s:%s", key, value)
                 return
@@ -324,8 +345,8 @@ class Span(object):
 
         try:
             self._meta[key] = stringify(value)
-            if key in self.metrics:
-                del self.metrics[key]
+            if key in self._metrics:
+                del self._metrics[key]
         except Exception:
             log.warning("error setting tag %s, ignoring it", key, exc_info=True)
 
@@ -352,7 +373,7 @@ class Span(object):
         """Return the given tag or None if it doesn't exist."""
         return self._meta.get(key, None)
 
-    def _get_tags(self):
+    def get_tags(self):
         # type: () -> _MetaDictType
         """Return all tags."""
         return self._meta.copy()
@@ -368,6 +389,7 @@ class Span(object):
 
     @removed_property(
         message="Use Span.set_tag, Span.set_tags or Span.get_tag methods instead.",
+        category=DDTraceDeprecationWarning,
         removal_version="1.0.0",
     )
     def meta(self):
@@ -377,10 +399,12 @@ class Span(object):
     def meta(self, value):
         self._meta = value
 
+    @remove(message="Use Span.set_tag instead.", category=DDTraceDeprecationWarning, removal_version="1.0.0")
     def set_meta(self, k, v):
         # type: (_TagNameType, NumericType) -> None
         self.set_tag(k, v)
 
+    @remove(message="Use Span.set_tags.", category=DDTraceDeprecationWarning, removal_version="1.0.0")
     def set_metas(self, kvs):
         # type: (_MetaDictType) -> None
         self.set_tags(kvs)
@@ -416,7 +440,19 @@ class Span(object):
 
         if key in self._meta:
             del self._meta[key]
-        self.metrics[key] = value
+        self._metrics[key] = value
+
+    @removed_property(
+        message="Use Span.get_metric or Span.set_metric instead.",
+        category=DDTraceDeprecationWarning,
+        removal_version="1.0.0",
+    )
+    def metrics(self):
+        return self._metrics
+
+    @metrics.setter  # type: ignore[no-redef]
+    def metrics(self, value):
+        self._metrics = value
 
     def set_metrics(self, metrics):
         # type: (_MetricDictType) -> None
@@ -426,8 +462,15 @@ class Span(object):
 
     def get_metric(self, key):
         # type: (_TagNameType) -> Optional[NumericType]
-        return self.metrics.get(key)
+        """Return the given metric or None if it doesn't exist."""
+        return self._metrics.get(key)
 
+    def get_metrics(self):
+        # type: () -> _MetricDictType
+        """Return all metrics."""
+        return self._metrics.copy()
+
+    @remove(message="Use getters for ``Span`` attributes instead.", version="1.0.0")
     def to_dict(self):
         # type: () -> Dict[str, Any]
         d = {
@@ -456,8 +499,8 @@ class Span(object):
         if self._meta:
             d["meta"] = self._meta
 
-        if self.metrics:
-            d["metrics"] = self.metrics
+        if self._metrics:
+            d["metrics"] = self._metrics
 
         if self.span_type:
             d["type"] = self.span_type
@@ -508,7 +551,7 @@ class Span(object):
         self._remove_tag(ERROR_TYPE)
         self._remove_tag(ERROR_STACK)
 
-    @deprecated(message="Span.pprint will be removed.", version="1.0.0")
+    @removed_property(category=DDTraceDeprecationWarning, removal_version="1.0.0")
     def pprint(self):
         return self._pprint()
 
@@ -528,7 +571,7 @@ class Span(object):
             ("duration", self.duration),
             ("error", self.error),
             ("tags", dict(sorted(self._meta.items()))),
-            ("metrics", dict(sorted(self.metrics.items()))),
+            ("metrics", dict(sorted(self._metrics.items()))),
         ]
         return " ".join(
             # use a large column width to keep pprint output on one line
